#include "crab.h"

// this version should never get called
crab::crab() {
	xcor = uniform_random() * SIZE_X + MIN_X;
	ycor = uniform_random() * SIZE_Y + MIN_Y;
	zcor = uniform_random() * SIZE_Z + MIN_Z;
	dx = 1.0;
	dy = 0.0;
	dz = 0.0;
	gene1 = 7.0;
	Assert(false, "crab initialization without gene");
}
crab::crab(double g) {
	xcor = uniform_random() * SIZE_X + MIN_X;
	ycor = uniform_random() * SIZE_Y + MIN_Y;
	zcor = uniform_random() * SIZE_Z + MIN_Z;
	dx = 1.0;
	dy = 0.0;
	dz = 0.0;
	gene1 = g;
	// Implement reflective boundary conditions
	//if (gene1 > GENE1_MAX) { gene1 = GENE1_MAX*GENE1_MAX / gene1; }
	//if (gene1 < GENE1_MIN) { gene1 = GENE1_MIN*GENE1_MIN / gene1; }
	Assert( GENE1_MAX > gene1, "gene too big");
	Assert( GENE1_MIN < gene1, "gene too small");
}
crab::~crab(){};

////////private////////////
inline double crab::min(double x, double y) {
	return (x < y) ? x : y;
}
inline double crab::min_mag(double x, double y) {
	return (fabs(x) < fabs(y)) ? x : y;
}
inline double crab::min_mag(double x, double y, double z) { 
	x = (fabs(x) < fabs(y)) ? x : y;
	x = (fabs(x) < fabs(z)) ? x : z;
	return x;
}

void crab::wrap_coordinates(void) {
	while (xcor > MAX_X) { xcor -= SIZE_X; }
	while (xcor < MIN_X) { xcor += SIZE_X; }
	while (ycor > MAX_Y) { ycor -= SIZE_Y; }
	while (ycor < MIN_Y) { ycor += SIZE_Y; }
	while (zcor > MAX_Z) { zcor -= SIZE_Z; }
	while (zcor < MIN_Z) { zcor += SIZE_Z; }
}

////////public////////////
double crab::get_x(void) { return xcor; }
double crab::get_y(void) { return ycor; }
double crab::get_z(void) { return zcor; }
double crab::get_gene1(void) { return gene1; }
double crab::set_gene1(double x) { gene1 = x; }

void crab::reset_galileon(void) {
	xcor = uniform_random() * SIZE_X + MIN_X;
	ycor = uniform_random() * SIZE_Y + MIN_Y;
	zcor = uniform_random() * SIZE_Z + MIN_Z;
}

void crab::walk_forward(double distance) {
	xcor += dx * distance;
	ycor += dy * distance;
	zcor += dz * distance;
	wrap_coordinates();
}

void crab::think_about_herd(crab* (herd_pointer[]),
			const int current_crab,
			const int NUMBER) {
	double x_resultant = 0;
	double y_resultant = 0;
	double z_resultant = 0;
	double weight, tmpx, tmpy, tmpz, d,m;
	for (long i=0; i<NUMBER; ++i ) {
		if ( i == current_crab ) continue; 
		crab* neighbor = herd_pointer[i];
		// Get x-distance and wrap for toroidal universe.
		tmpx = neighbor->get_x() - xcor;
		tmpx = min_mag( tmpx - SIZE_X, tmpx, tmpx + SIZE_X);
		// Get y-distance and wrap for toroidal universe.
		tmpy = neighbor->get_y() - ycor;
		tmpy = min_mag( tmpy - SIZE_Y, tmpy, tmpy + SIZE_Y);
		// Get z-distance and wrap for toroidal universe.
		tmpz = neighbor->get_z() - zcor;
		tmpz = min_mag( tmpz - SIZE_Z, tmpz, tmpz + SIZE_Z);

		d = sqrt( tmpx*tmpx + tmpy*tmpy + tmpz*tmpz);
		//if ( d < 1e-12) continue;  // prevent bad values
		Assert( d > 1e-12, "Distance to crab is too small");
		/*******Key weighting function**************/
		weight = 1/( 1+pow(d/RADIUS, gene1));
		/******End of weight calculation ******/
		/*** Alternative forms of weighting ****/
		//weight = pow( 1 + m*100, -gene1);
		//weight = pow( 1 + m/gene1, -gene1);
		//weight = exp( 1 - pow(1+gene1,m));
		//weight = exp( - pow(m, gene1));

		x_resultant += tmpx * weight / d;
		y_resultant += tmpy * weight / d;
		z_resultant += tmpz * weight / d;
	}
	weight = 
		sqrt( x_resultant*x_resultant + y_resultant*y_resultant + z_resultant*z_resultant);
	//Assert(1e-14<weight, "normalizing a zero vector");
	dx = x_resultant/weight;
	dy = y_resultant/weight;
	dz = z_resultant/weight;
}
