#!/bin/tcsh -f

# Experiment 5.6: broad genotype bounds

# A slower mutation rate is used than in experiment 5
# The Standard temporal resolution is used.
# The Initial genotype is set higher than usual, because
# of the larger upper bound on the genotype.

set niceness=" nice +8 "
set repeats=8
# 8 nodes x 8 repeats = 64 trials, 
set UP_NODES=(	ruby wallula moclips \
				steptoe brewster poulsbo \
				nighthawk conconully )

set prefix="data";
set steps=20 ; # The standard.  Needs to be a variable for vel calculations.
set command="evol_crabs -n75 -r0.002 -m0.5 -g7.0 -e$steps -p5001 -P20 -OA -T -S "
echo "# " $command
set distances=( 0.0125 0.05 0.1 0.2 0.4 0.8 2.0 4.0 )
set testing=" "
### For Testing purposes
set repeats=3
set UP_NODES=(	ruby moclips conconully )
set distances=( 0.0125 4.0 )
set testing="-n10 -p7 -P3"

foreach x ( $distances )
	if ( { test -e $prefix$x } ) then
		echo $prefix$x exists, so I am exitting
		exit 1 
	endif
end

nohup
touch start_time
foreach x ( $distances )
	set vel=`echo $x \/ $steps  | bc -l`;
	echo "# " $x at $steps steps gives $vel per step
	$command -v$vel $testing -Ho > $prefix$x
	foreach n ( $UP_NODES )
		ssh $n $niceness repeat $repeats $command -v$vel $testing >> $prefix$x.$n &;
		sleep 1s
	end
	wait
	foreach n ( $UP_NODES )
		echo '' >> $prefix$x.$n
		cat $prefix$x.$n >> $prefix$x
	end
end
chmod 400 $prefix*
