#include <stdio.h>
#include <iostream>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include "flag_collection.h"
#include "myassert.h"
#include "mymath.h"

using namespace std;

#include "global.h"


flag_collection::flag_collection(int argc, char** argv) :
  tabs(false),
  newlines(true),
  scientific(false),
  random_seed(getrandomseed()),
  print_period(199),
  output(AVE),
  header(NONE),
  predation_events(200),
  predation_discards(0),
  gene1_start(7.0),
  number_of_crabs(75),
  escape_velocity(.01),
  escape_time(20),
  mutation_rate(1.0),
  radius(-1.0)
{
	const char* message[] = {
		"evol_crab",
		"Purpose: stochastic evolution of herd behavior",
		"Usage: evol_crab <options>",
		"Options:",
		"\t-h\tdisplays this message.",
		"\t-?\tdisplays this message.",
		"\t-D\tdisplays default parameter values, I hope.",
		"\t-P#\tPrint output every so-many attacks.",
		"\t-S#\tPrint output in scientific notation.",
		"\t-O*\tOutput style",
		"\t\t-OA\t Average, Standard Deviation, Min, Max",
		"\t\t\tIf used with -T, only the average is printed.",
		"\t\t-OD\t Gene values",
		"\t\t-OP\t Crab positions before predation",
		"\t-H*\tHeader style",
		"\t\t-Ha\tOutput a header with parameter values.",
		"\t\t-Ho\tOnly output a header with parameter values.",
		"\t\t-Hr\tOutput a header with only the random seed.",
		"\t-T\tDelimit output by tabs instead of newlines.",
		"\t-R#\tInstead of a random seed, use this number",
		"\t-p#\tNumber of predation events in experiment",
		"\t-d#\tNumber of predation events to discard before starting",
		"\t-g#\tAssign an initial value to gene 1",
		"\t-n#\tNumber of crabs in herd",
		"\t-v#\tEscape velocity of a crab",
		"\t-e#\tEscape time between warning and attack",
		"\t-m#\tMutation rate(orders of magnitude, proportional",
		"\t\tto Coefficient of Variation)",
		"\t-r#\tUse this fixed radius for half-saturation level",
		" ",
		NULL
	};

	//assume all options occur right after the
	//programs name, on the command line
	short mark;
	for (mark=1; argv[mark] && argv[mark][0]=='-';mark++) {
		switch (argv[mark][1]) {
		  case 'h':
		  case '?':
			for (short i=0; message[i]; i++) {
				cerr << message[i] << endl;
			}
		  case 'D':
			cerr << "Default parameter values" << endl
				 << "\tEvolution on a 1-unit square toris." << endl
				 << "\tPredations: " << predation_events << endl
				 << "\tDiscarded Predations: " << predation_discards << endl
				 << "\tNumber of Crabs: " << number_of_crabs << endl
				 << "\tEscape Velocity: " << escape_velocity << endl
				 << "\tEscape Time: " << escape_time << endl
				 << "\tInitial Mean Gene: " << gene1_start << endl
				 << "\tMax Gene: " << GENE1_MAX << endl
				 << "\tMin Gene: " << GENE1_MIN << endl
				 << "\tMutation Rate: " << mutation_rate << endl;
			exit(0);
			break;
		  case 'S':
			scientific = true;
			break;
		  case 'T':
			tabs = true;
			newlines = false;
			break;
		  case 'H':
			switch (argv[mark][2]) {
				case 'a': header = REGULAR; break;
				case 'o': header = ONLY; break;
				case 'r': header = SEED; break;
				default: Assert(false, "Header option");
			}
			break;
		  case 'R':
			Assert(sscanf(argv[mark],"-R%d",&random_seed),
				   "Bad seed value");
			break;
		  case 'P':
			Assert(sscanf(argv[mark],"-P%ld",&print_period),
				   "Bad print period");
			break;
		  case 'O':
			switch (argv[mark][2]) {
				case 'A': output = AVE; break;
				case 'D': output = DIST; break;
				case 'P': output = POS; break;
				default: Assert(false, "Output option");
			}
			break;
		  case 'p':
			Assert(sscanf(argv[mark],"-p%ld",&predation_events),
				   "Bad number of predation events");
			break;
		  case 'd':
			Assert(sscanf(argv[mark],"-d%ld",&predation_discards),
				   "Bad number of predation discards");
			break;
		  case 'g':
			Assert(sscanf(argv[mark],"-g%lf",&gene1_start),
				   "Bad gene1 input");
			break;
		  case 'v':
			Assert(sscanf(argv[mark],"-v%lf",&escape_velocity),
				   "Bad escape velocity");
			break;
		  case 'e':
			Assert(sscanf(argv[mark],"-e%ld",&escape_time),
				   "Bad escape time");
			break;
		  case 'm':
			Assert(sscanf(argv[mark],"-m%lf",&mutation_rate),
				   "Bad mutation rate");
			break;
		  case 'r':
			Assert(sscanf(argv[mark],"-r%lf",&radius),
				   "half-saturation radius");
			break;
		  case 'n':
			Assert(sscanf(argv[mark],"-n%ld",&number_of_crabs),
				   "Bad number of crabs");
			break;
		  default:
			cerr << "Invalid option: " << argv[mark] << endl;
		}
	}
	Assert(print_period <= predation_events,"Print period too long");
	switch (header) {
		case ONLY:
		case REGULAR:
			cout << "##### header #######" << endl
			 << "#\tEvolution on a 1-unit square toris." << endl
			 << "#\tPredations: " << predation_events << endl
			 << "#\tPredation discards: " << predation_discards << endl
			 << "#\tNumber of Crabs: " << number_of_crabs << endl
			 << "#\tEscape Velocity: " << escape_velocity << endl
			 << "#\tEscape Time: " << escape_time << endl
			 << "#\tInitial Mean Gene: " << gene1_start << endl
			 << "#\tMax Gene: " << GENE1_MAX << endl
			 << "#\tMin Gene: " << GENE1_MIN << endl
			 << "#\tMutation Rate: " << mutation_rate << endl
			 << "#\tHalf saturation radius(-1 means standard): " << radius << endl
			 << "#\tOutput period(iteration units): " << print_period << endl
			 << "#\tRandom seed: " << random_seed << endl
			 << "##### end header #########" << endl;
			if (ONLY == header) exit(0);
			break;
		case SEED:
			cout << "#\tRandom seed:\t" << random_seed << endl;
			break;
		case NONE: break;
		default: Assert(false, "header error 3");  break;
	}
	return;
}
/*************************************************************************
*                                                                        *
*       input_file = argv[mark];                                         *
*                                                                        *
*       if (input_file == NULL) {                                        *
*           cerr << "Reading standard input? not implemented" << endl;   *
*           exit(0);                                                     *
*       } else if ( (input = fopen(input_file,"r")) == NULL) {           *
*           cerr << "Bad file name!" << endl;                            *
*           exit(0);                                                     *
*       }                                                                *
*                                                                        *
*************************************************************************/
