#include "crab.h"

// this version should never get called
crab::crab() {
	xcor = uniform_random() * SIZE_X + MIN_X;
	ycor = uniform_random() * SIZE_Y + MIN_Y;
	heading = uniform_random()*2*M_PI;
	dx = cos(heading);
	dy = sin(heading);
	gene1 = 7.0;
	gene2 = 1.0;
	Assert(false, "crab initialization without gene");
}
crab::crab(double g) {
	xcor = uniform_random() * SIZE_X + MIN_X;
	ycor = uniform_random() * SIZE_Y + MIN_Y;
	heading = uniform_random()*2*M_PI;
	dx = cos(heading);
	dy = sin(heading);
	gene1 = g;
	// Implement reflective boundary conditions
	//if (gene1 > GENE1_MAX) { gene1 = GENE1_MAX*GENE1_MAX / gene1; }
	//if (gene1 < GENE1_MIN) { gene1 = GENE1_MIN*GENE1_MIN / gene1; }
	Assert( GENE1_MAX > gene1, "gene too big");
	Assert( GENE1_MIN < gene1, "gene too small");
}
crab::~crab(){};

////////private////////////
inline double crab::min(double x, double y) {
	return (x < y) ? x : y;
}
inline double crab::min_mag(double x, double y) {
	return (fabs(x) < fabs(y)) ? x : y;
}
inline double crab::min_mag(double x, double y, double z) { 
	x = (fabs(x) < fabs(y)) ? x : y;
	x = (fabs(x) < fabs(z)) ? x : z;
	return x;
}

void crab::wrap_coordinates(void) {
	while (xcor > MAX_X) { xcor -= SIZE_X; }
	while (xcor < MIN_X) { xcor += SIZE_X; }
	while (ycor > MAX_Y) { ycor -= SIZE_Y; }
	while (ycor < MIN_Y) { ycor += SIZE_Y; }
}
double crab::angle(double dx, double dy) {
	double tmp1 = acos(dx);
	double tmp2 = asin(dy);
	double theta = ( tmp2 > 0) ? tmp1 : 2*M_PI - tmp1;
	return theta;
}

// Return the distance to a point for this crab
inline double crab::distance(double x, double y) {
	double a = (x-xcor)*(x-xcor) + (y-ycor) *( y-ycor);
	Assert(a>0, "Zero distance problem");
	return sqrt(a);
}

////////public////////////
double crab::get_x(void) { return xcor; }
double crab::get_y(void) { return ycor; }
double crab::get_gene1(void) { return gene1; }
double crab::get_gene2(void) { return gene2; }
double crab::set_gene1(double x) { gene1 = x; }
double crab::set_gene2(double x) { gene2 = x; }

void crab::reset_galileon(void) {
	xcor = uniform_random() * SIZE_X + MIN_X;
	ycor = uniform_random() * SIZE_Y + MIN_Y;
	heading = uniform_random()*2*M_PI;
	dx = cos(heading);
	dy = sin(heading);
}
void crab::wobble(double max_shift) {
	double rval = 2 * max_shift * uniform_random();
	heading += max_shift - rval;
	dx = cos(heading);
	dy = sin(heading);
}
void crab::walk_forward(double distance) {
	xcor += dx * distance;
	ycor += dy * distance;
	wrap_coordinates();
}
void crab::wander(double distance) {
	wobble(	WOBBLE_ANGLE );
	walk_forward(distance);
}
void crab::think_about_herd(crab* (herd_pointer[]),
			const int current_crab,
			const int NUMBER) {
	double x_resultant = 0;
	double y_resultant = 0;
	double weight, tmpx, tmpy, d,m;
	for (long i=0; i<NUMBER; ++i ) {
		if ( i == current_crab ) continue; 
		crab* neighbor = herd_pointer[i];
		// Get x-distance and wrap for toroidal universe.
		tmpx = neighbor->get_x() - xcor;
		tmpx = min_mag( tmpx - SIZE_X, tmpx, tmpx + SIZE_X);
		// Get y-distance and wrap for toroidal universe.
		tmpy = neighbor->get_y() - ycor;
		tmpy = min_mag( tmpy - SIZE_Y, tmpy, tmpy + SIZE_Y);

		d = hypot(tmpx,tmpy);
		//if ( d < 1e-12) continue;  // prevent bad values
		//Assert( d > 1e-12, "Distance to crab is too small");
		/*******Key weighting function**************/
		weight = 1/( 1+pow(d/RADIUS, gene1));
		/******End of weight calculation ******/
		/*** Alternative forms of weighting ****/
		//weight = pow( 1 + m*100, -gene1);
		//weight = pow( 1 + m/gene1, -gene1);
		//weight = exp( 1 - pow(1+gene1,m));
		//weight = exp( - pow(m, gene1));

		x_resultant += tmpx * weight / d;
		y_resultant += tmpy * weight / d;
	}
	weight = hypot(x_resultant, y_resultant);
	//Assert(1e-14<weight, "normalizing a zero vector");
	//if (1e-12<weight) {
		dx = x_resultant/weight;
		dy = y_resultant/weight;
		heading = angle(dx,dy);
	//}
	//Assert( dx*dx+dy*dy < 1.01, "hat{z}^star not a unit vector");
}
