#include "mymath.h"
#include "myassert.h"

#include "global.h"

extern double RUNNING_STEP;
extern double RADIUS;
extern double MUTATION_RATE;

class crab {
  private:
	double dx;  // X component of unit direction vector
	double dy;  // Y component of unit direction vector
	double heading; // An angle in radians
	double xcor; // X component of position
	double ycor; // Y component of position

	// gene1 is a parameter used in weighting the location of others
	// when deciding where to flee to.
	double gene1;

	// Orignially,
	// gene2 is a parameter used to decide what to do with the
	// herd resultant produced using gene1.  If gene2 > 0, the
	// crab will flee toward the herd.  If gene2<0, the crab
	// will flee away from the herd.
	// Currently, gene2 is unused.
	double gene2;

	void wrap_coordinates(void);
	double angle(double dx, double dy);
	inline double magnitude(double x, double y);
	inline double distance(double x, double y);
	inline double min(double x, double y);
	inline double min_mag(double x, double y);
	inline double min_mag(double x, double y, double z);
  public:
	crab();
	crab(double g);
	~crab();
	
	double set_gene1(double x);
	double set_gene2(double x);

	double get_x(void);
	double get_y(void);
	double get_gene1(void);
	double get_gene2(void);

	void reset_galileon(void);
	void wobble(double max_shift);
	void walk_forward(double distance);
	void wander(double distance);
	void think_about_herd(crab* (herd_pointer[]), const int current_crab, const int NUMBER);
	void flight_manuever(crab* (herd_pointer[]));
};
