/*
* This function takes a set of parameters from matlab
* and calculates a time series of positions for individuals.
* The outputs are returned as 2 matrices, one with the x-coordinates
* for each time, and one for the y coordinates of each individual
* for each time. 
*
*/
#include <ctype.h>
#include <time.h>
#include "mex.h"
#include "mymath.h"
#include "myassert.cc"

#include "crab.h"
const char* error_message[] = {
	"First input should be a counting number",
	"Second input should be a counting number",
	"Third input should be a counting number",
	"Fourth input should be a real number",
};

//double RUNNING_STEP = 0.20;
double MUTATION_RATE	= 1.0;
double RADIUS = 0.002;
//long TIME_FOR_ESCAPE	= 20;

//	INPUTS
// nrhs - number of right hand side arguements
// prhs - pointer to right hand side arguements
//	OUTPUTS
// nlhs - number of left hand side arguements
// plhs - pointer to left hand side arguements
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
	long mrows, ncols;
	long TOTAL_CRABS, TIME_BEFORE_ATTACK, TOTAL_TIME;
	double TOTAL_DISTANCE;
	const long TIME_FOR_ESCAPE = 20;
	double GENE1;
	Assert(nlhs == 2, "Should return 2 matrices");
	Assert(nrhs == 4, "takes .(total crabs, time before attack, distance after warning, gene)");
	
	for (short i=0; i<4; i++) {
		mrows = mxGetM(prhs[i]);
		ncols = mxGetN(prhs[i]);
		Assert( mxIsDouble(prhs[i]) &&
			   !mxIsComplex(prhs[i]) &&
			   mrows == 1 && ncols == 1, error_message[i]);
	}
	TOTAL_CRABS			= long(mxGetScalar(prhs[0]));
	TIME_BEFORE_ATTACK	= long(mxGetScalar(prhs[1]));
	TOTAL_DISTANCE		= double(mxGetScalar(prhs[2]));
	GENE1 				= mxGetScalar(prhs[3]);
	Assert(GENE1>0,"bad gene");

	TOTAL_TIME = TIME_BEFORE_ATTACK + TIME_FOR_ESCAPE;
	double RUNNING_STEP = TOTAL_DISTANCE/double(TIME_FOR_ESCAPE);

	plhs[0] = mxCreateDoubleMatrix(TOTAL_TIME, TOTAL_CRABS, mxREAL);
	plhs[1] = mxCreateDoubleMatrix(TOTAL_TIME, TOTAL_CRABS, mxREAL);

	double* x = mxGetPr(plhs[0]);
	double* y = mxGetPr(plhs[1]);

	crab* herd[TOTAL_CRABS+1];
	herd[TOTAL_CRABS]=NULL;
	double herd_distances[TOTAL_CRABS*TOTAL_CRABS/2 + TOTAL_CRABS];
	for (long i=0; i<TOTAL_CRABS; i++) {
		herd[i] = new crab(GENE1);
	}

	Assert(MAX_X - MIN_X > 0, "bad boundaries");
	Assert(MAX_Y - MIN_Y > 0, "bad boundaries");
	//Assert( srandom((int) time(NULL)) == 0, "srandom failed");
	srandom((int) time(NULL));

	long t;
	for (t=0; t<TIME_BEFORE_ATTACK; t++) {
		for (long i=0; herd[i]!=NULL; i++) {
			herd[i]->wander(RUNNING_STEP);
			x[i*TOTAL_TIME + t] = herd[i]->get_x();
			y[i*TOTAL_TIME + t] = herd[i]->get_y();
		}
	}
	for (; t<TOTAL_TIME; t++) {
		for (long i=0; herd[i]!=NULL; i++) {
			herd[i]->think_about_herd(herd,i,TOTAL_CRABS);
		}
		for (long i=0; herd[i]!=NULL; i++) {
			herd[i]->walk_forward(RUNNING_STEP);
			x[i*TOTAL_TIME + t] = herd[i]->get_x();
			y[i*TOTAL_TIME + t] = herd[i]->get_y();
		}
	}
}
